/***************************************************************************
 *                                                                         *
 *   YANN - Yet Another Neural Network                                     *
 *   =================================                                     *
 *                                                                         *
 *   Copyright (C) Juan Pedro Bolivar Puente 2008                          *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <fstream>

#include "pipeline.h"
#include "perceptron.h"
#include "inputtask.h"
#include "outputtask.h"
#include "argparser.h"
#include "network.h"

using namespace std;
using namespace yann;

void runTestNetwork ()
{
    Pipeline layer_in;
    Pipeline layer_out;
    Pipeline layer_hid;
    
    Perceptron node_a;
    Perceptron node_b;
    Perceptron node_out;
    Input input_a;
    Input input_b;

    OutputTask out_task (&cout);
    InputTask in_task (&cin);

    layer_in.addTask (&in_task);

    layer_hid.addTask (&node_a);
    layer_hid.addTask (&node_b);
    
    layer_out.addTask (&node_out);
    layer_out.addTask (&out_task);

    node_out.connect (Perceptron::Link (&node_a, 0.4));
    node_out.connect (Perceptron::Link (&node_b, 0.6));
    node_b.connect (Perceptron::Link (&input_a, 0.2));
    node_b.connect (Perceptron::Link (&input_b, -0.2));
    node_a.connect (Perceptron::Link (&input_a, -0.6));
    node_a.connect (Perceptron::Link (&input_b, 0.6));

    in_task.addInput (&input_a);
    in_task.addInput (&input_b);
    out_task.addOutput (&node_out);
	
    layer_in.addNext (&layer_hid);
    layer_hid.addNext (&layer_out);

    layer_in.start ();
    layer_hid.start ();
    layer_out.start ();
    
    layer_out.wait ();
}

void showHelp ()
{
    cout << "yann - yet another neural network.\n"
	 << " (c) 2008 Juan Pedro Bolívar Puente.\n"
	 << "\n"
	 << "uso:\n"
	 << "   yann fichero_de_red [opciones]\n"
	 << "\n"
	 << "opciones:\n"
	 << "  -i, --input <fichero>   Lee la entrada desde fichero en lugar de desde\n"
	 << "                          la entrada estándar.\n"
	 << "  -o, --output <fichero>  Emite la salida sobre un fichero en lugar de\n"
	 << "                          sobre la salida estandar.\n"
	 << "  -h, --help              Muestra esta ayuda.\n";
}

int main (int argc, const char* argv[])
{
    ArgParser args;
    bool is_help = false;
    std::string in_file;
    std::string out_file;
    
    args.add ('h', "help", &is_help);
    args.add ('i', "input", &in_file);
    args.add ('o', "output", &out_file);
    args.parse (argc, argv);

    if (is_help) {
	showHelp ();
	return -1;
    }

    if (!args.hasFreeArgs ()) {
	cerr << "Debe especificar un fichero de definición de la red. Use -h para la ayuda." << endl;
	return -1;
    }

    istream* in_fstream = &cin;
    ostream* out_fstream = &cout;
    ifstream net_fstream (*args.begin ());

    if (!net_fstream.is_open ()) {
	cerr << "No se pudo abrir el fichero de la red: " << *args.begin () << endl;
	return -1;
    }
    
    Network network (net_fstream);

    net_fstream.close ();
    
    if (!in_file.empty ()) {
	in_fstream = new ifstream (in_file.c_str ());
	if (!((fstream*)in_fstream)->is_open()) {
	    cerr << "No se pudo abrir el fichero de entrada: " << in_file << endl;
	    return -1;
	}
    }

    if (!out_file.empty ()) {
	out_fstream = new ofstream (out_file.c_str ());
	if (!((fstream*)out_fstream)->is_open()) {
	    cerr << "No se pudo abrir el fichero de salida: " << out_file << endl;
	    return -1;
	}
    }

    network.run (*in_fstream, *out_fstream);

    if (!in_file.empty ()) {
	((fstream*)in_fstream)->close ();
	delete in_fstream;
    }

    if (!out_file.empty ()) {
	((fstream*)out_fstream)->close ();
	delete out_fstream;
    }
    
    return 0;
}
