/***************************************************************************
 *                                                                         *
 *   YANN - Yet Another Neural Network                                     *
 *   =================================                                     *
 *                                                                         *
 *   Copyright (C) Juan Pedro Bolivar Puente 2008                          *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                         *
 ***************************************************************************/

#ifndef YANN_PERCEPTRON_H
#define YANN_PERCEPTRON_H

#include "task.h"
#include "node.h"

#include <list>
#include <cmath>

namespace yann
{

class Perceptron : public Task,
		   public Node
{
public:
    struct Link
    {
	Node* node;
	Value weight;

	Link (Node* n, Value w) :
	    node(n), weight(w) {}
	
	bool operator == (const Link& l) {
	    return l.node == node;
	}
    };

    typedef Value(*function_t)(Value);

    static Value sigmoidFunction (Value val) {
	return 1.0 / (1 + std::exp (-val));
    };

    static Value thresholdFunction (Value val) {
	return val > 0.5 ? 1.0 : 0.0;
    }
    
private:
    
    /*
      We use a list to enhance efficiency on execution, but a map
      would be better to improbe efficiency when connections are changed
      quite often.
    */
    std::list<Link> m_input;
    Value m_bias;
    Value m_output;
    function_t m_act_func;
    function_t m_out_func;
    
public:
    Perceptron () :
	m_bias (0.0f),
	m_output (0.0f),
	m_act_func (Perceptron::sigmoidFunction),
	m_out_func (0)
	{}

    void setActivationFunction (function_t func) {
	m_act_func = func;
    }

    void setOutputFunction (function_t func) {
	m_out_func = func;
    }
    
    Value output () {
	return m_output;
    }
    
    void setBias (Value b) {
	m_bias = b;
    }

    void connect (Link l) {
	m_input.push_back (l);
    }
    
    void disconnect (Node* n) {
	m_input.remove (Link (n, 0));
    }

    bool update ();
};

} /* namespace yann */

#endif /* YANN_PERCEPTRON_H */
